import { Component, createResource } from 'solid-js';
import { peekCachedSignal } from '../../workers/cached-signals';
import RteIcon from '../../icons/directions-svgrepo-com.svg?component-solid';
import { useI18n } from '@solid-primitives/i18n';

import Tree from '../tree';
import { findAddress, getVillageOrTown } from '../../lib/osm';
import { getLength } from 'ol/sphere';
import { Geometry, LineString } from 'ol/geom';

interface Props {
  rteId: string;
}

const RteViewer: Component<Props> = ({ rteId }) => {
  const [t, { add, locale, dict }] = useI18n();

  const rte = peekCachedSignal({ id: rteId, method: 'getRte' });
  console.log({ caller: 'RteViewer', rteId, rte: rte() });
  const title = () => {
    return rte().name;
  };

  const [startAddress] = createResource(rte, (r: Rte) =>
    findAddress(r.rtept[0].$.lon, r.rtept[0].$.lat, locale)
  );
  const [endAddress] = createResource(rte, (r: Rte) =>
    findAddress(r.rtept.at(-1).$.lon, r.rtept.at(-1).$.lat, locale)
  );

  const formatLength = (length: number) => {
    if (length > 1000) {
      const l = length / 1000;
      return ` ${l.toLocaleString(undefined, {
        minimumFractionDigits: 2,
        maximumFractionDigits: 2,
      })} km`;
    }
    return ` ${length.toLocaleString(undefined, {
      minimumFractionDigits: 0,
      maximumFractionDigits: 0,
    })} m`;
  };

  const geometry = new LineString(
    rte().rtept.map((rtept: any) => [rtept.$.lon, rtept.$.lat])
  );

  const length = getLength(geometry, { projection: 'EPSG:4326' });

  return (
    <Tree
      title={
        <>
          <RteIcon fill='black' width='24' height='24' /> {title()}
        </>
      }
      content={
        <>
          <div>{`${formatLength(length)} ${t('from')} ${getVillageOrTown(
            startAddress()
          )} ${t('to')} ${getVillageOrTown(endAddress())} `}</div>
        </>
      }
      subTree={undefined}
    />
  );
};

export default RteViewer;
